@@ -692,25 +692,6 @@ skip_non_required:
 		return -1;
 	}
 	close(fd);
-
-	if (opt(bind_all_ind)) {
-		memset(path, 0, PATH_MAX);
-		snprintf(path, PATH_MAX, "%s/%d/bind_all", COMMS_DIR, nodeid);
-
-		fd = open(path, O_WRONLY);
-		if (fd < 0) {
-			log_error("%s: open failed: %d", path, errno);
-			return -1;
-		}
-
-		rv = do_write(fd, (void *)"1", strlen("1"));
-		if (rv < 0) {
-			log_error("%s: write failed: %d", path, errno);
-			close(fd);
-			return -1;
-		}
-		close(fd);
-	}
  out:
 	return 0;
 }
