@@ -1886,6 +1886,7 @@ static void send_protocol(struct protocol *proto)
 	protocol_out(pr);
 
 	_send_message(cpg_handle_daemon, buf, len, DLM_MSG_PROTOCOL);
+	free(buf);
 }
 
 int set_protocol(void)
@@ -2079,7 +2080,7 @@ int receive_run_reply(struct dlm_header *hd, int len)
 		return 0;
 
 	if (len != sizeof(struct run_reply)) {
-		log_debug("receive_run_reply %s bad len %s expect %d",
+		log_debug("receive_run_reply %s bad len %d expect %zu",
 			  rep->uuid, len, sizeof(struct run_reply));
 		run->info.reply_count++;
 		run->info.need_replies--;
@@ -2526,6 +2527,15 @@ int setup_cpg_daemon(void)
 	return -1;
 }
 
+static void stop_lockspaces(void)
+{
+	struct lockspace *ls;
+
+	list_for_each_entry(ls, &lockspaces, list) {
+		cpg_stop_kernel(ls);
+	}
+}
+
 void close_cpg_daemon(void)
 {
 	struct lockspace *ls;
@@ -2533,8 +2543,11 @@ void close_cpg_daemon(void)
 	struct cpg_name name;
 	int i = 0;
 
-	if (!cpg_handle_daemon)
+	if (!cpg_handle_daemon) {
+		stop_lockspaces();
 		return;
+	}
+
 	if (cluster_down)
 		goto fin;
 
@@ -2555,6 +2568,8 @@ void close_cpg_daemon(void)
 		log_error("daemon cpg_leave error %d", error);
  fin:
 	list_for_each_entry(ls, &lockspaces, list) {
+		/* stop kernel ls lock activity before configfs cleanup */
+		cpg_stop_kernel(ls);
 		if (ls->cpg_handle)
 			cpg_finalize(ls->cpg_handle);
 	}
